SystemOrganization addCategory: #TusLibros!

Object subclass: #Cart
	instanceVariableNames: 'books catalog'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

Cart class
	instanceVariableNames: 'size'!

Cart class
	instanceVariableNames: 'size'!

!Cart class methodsFor: 'class initialization' stamp: 'JuanManuelTastzian 5/29/2013 18:30'!
newEmptyCart
	^self new withoutBooks! !

!Cart class methodsFor: 'error handling' stamp: 'JuanManuelTastzian 5/29/2013 20:54'!
productNotInCatalog
	^ 'Error: The requested product is not present in the catalog'.! !

!Cart class methodsFor: 'class initialization' stamp: 'JuanManuelTastzian 5/29/2013 20:41'!
withCatalog: aCatalog 
	^ self new catalog: aCatalog.! !

!Cart methodsFor: 'accessing' stamp: 'JuanManuelTastzian 5/29/2013 20:53'!
add: aBook 
	(catalog includes: aBook) ifTrue: [books add: aBook] ifFalse: [ Error signal: 'The requested product is not present in the catalog' ]
	! !

!Cart methodsFor: 'accessing' stamp: 'JuanManuelTastzian 5/29/2013 18:36'!
books
	^ books! !

!Cart methodsFor: 'accessing' stamp: 'JuanManuelTastzian 5/29/2013 20:43'!
catalog: aCatalog 
	books := Bag new.
	catalog := aCatalog.! !

!Cart methodsFor: 'accessing' stamp: 'JuanManuelTastzian 5/29/2013 19:16'!
includes: aBook 
	^books includes: aBook! !

!Cart methodsFor: 'accessing' stamp: 'JuanManuelTastzian 5/29/2013 20:04'!
isEmpty
	^ self size = 0! !

!Cart methodsFor: 'accessing' stamp: 'JuanManuelTastzian 5/29/2013 19:26'!
numberOfUnitsAddedOf: aBook 
	^(books occurrencesOf: aBook)! !

!Cart methodsFor: 'as yet unclassified' stamp: 'JuanManuelTastzian 5/29/2013 18:38'!
size
	^books size! !

!Cart methodsFor: 'accessing' stamp: 'JuanManuelTastzian 5/29/2013 20:27'!
withoutBooks
	books := Bag new.
	catalog := Bag new.! !

Object subclass: #Cashier
	instanceVariableNames: 'priceList'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!Cashier class methodsFor: 'message handling' stamp: 'JuanManuelTastzian 6/3/2013 19:12'!
bookNotInPriceList
	^ 'Error: Cannot checkout books that are not in the price list'! !

!Cashier class methodsFor: 'error handling' stamp: 'JuanManuelTastzian 5/29/2013 21:20'!
cartIsEmpty
	^ 'Error: Cannot checkout an empty cart'.! !

!Cashier class methodsFor: 'error handling' stamp: 'JuanManuelTastzian 6/3/2013 20:36'!
expiredCreditCard
	^ 'Error: Cannot checkout payment with expired credit card'! !

!Cashier class methodsFor: 'instance creation' stamp: 'SebaTorres 6/3/2013 02:54'!
withPriceList: aPriceList 
	^self new withPriceList: aPriceList.! !

!Cashier methodsFor: 'as yet unclassified' stamp: 'SebaTorres 6/3/2013 03:24'!
checkout: aCart
	(aCart isEmpty) ifTrue: [ Error signal: 'Cannot checkout an empty cart' ].
	(aCart books asSet = priceList keys asSet) ifFalse: [ Error signal: 'Cannot checkout books that are not in the price list' ].
	^aCart books detectSum: [ :book | priceList at: book ].! !

!Cashier methodsFor: 'as yet unclassified' stamp: 'JuanManuelTastzian 6/3/2013 20:43'!
checkout: aCart with: aCreditCard
	(aCart isEmpty) ifTrue: [ Error signal: 'Cannot checkout an empty cart' ].
	(aCreditCard isExpired) ifTrue: [ Error signal: 'Cannot checkout payment with expired credit card' ].
	(aCart books asSet = priceList keys asSet) ifFalse: [ Error signal: 'Cannot checkout books that are not in the price list' ].
	^aCart books detectSum: [ :book | priceList at: book ].! !

!Cashier methodsFor: 'accessing' stamp: 'SebaTorres 6/3/2013 02:56'!
withPriceList: aPriceList 
	priceList := aPriceList.! !

Object subclass: #CreditCard
	instanceVariableNames: 'expireDate isExpired'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!CreditCard class methodsFor: 'class initialization' stamp: 'JuanManuelTastzian 6/3/2013 20:31'!
newExpiredCreditCard
	^self new expiredCreditCard! !

!CreditCard methodsFor: 'accessing' stamp: 'JuanManuelTastzian 6/3/2013 21:07'!
expiredCreditCard
	expireDate := (Date today addDays: (-1)).
	isExpired := (Date today < expireDate)! !

!CreditCard methodsFor: 'accessing' stamp: 'JuanManuelTastzian 6/3/2013 21:07'!
isExpired
	^ isExpired! !

TestCase subclass: #TusLibrosTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!TusLibrosTest methodsFor: 'Cart Instances' stamp: 'JuanManuelTastzian 5/29/2013 18:23'!
emptyCart
	^Cart newEmptyCart! !

!TusLibrosTest methodsFor: '1 - Cart Test' stamp: 'JuanManuelTastzian 5/29/2013 20:01'!
test1CartShouldBeEmptyAfterCreation
	self assert: self emptyCart isEmpty! !

!TusLibrosTest methodsFor: '2 - Cashier Test' stamp: 'JuanManuelTastzian 5/29/2013 21:19'!
test1cantCheckoutEmptyCart

	| cashier |
	cashier := Cashier new.
	
	self should: [cashier checkout: (self emptyCart) ]
		raise: Error 
		withExceptionDo: [ :anError | 
			self assert: anError description = Cashier cartIsEmpty ]! !

!TusLibrosTest methodsFor: '1 - Cart Test' stamp: 'JuanManuelTastzian 5/29/2013 20:57'!
test2CartSizeShouldBeGreaterThan0AfterAddingABookToIt
	| cart book catalog |
	
	book := 'aBook'.
	catalog := Bag new.
	catalog add: book.
	cart := Cart withCatalog: catalog.
	
	cart add: book.
	
	self assert: (cart size > 0).! !

!TusLibrosTest methodsFor: '2 - Cashier Test' stamp: 'SebaTorres 6/3/2013 03:24'!
test2cashierChecksOutCorrectly

	"Este test se refiere a que el carrito hace bien el cobro de los libros?"

	| cashier aBook catalog priceList anotherBook cart priceBook priceAnotherBook |

	aBook := 'aBook'.
	anotherBook := 'anotherBook'.

	catalog := Bag new.
	catalog add: aBook.
	catalog add: anotherBook .

	"Para crear un diccionario se le asignan asociaciones"
	priceBook := Association key: aBook value: 1.
	priceAnotherBook := Association key: anotherBook value: 3.

	priceList := Dictionary new.
	priceList add: priceBook.
	priceList add: priceAnotherBook.

	cashier := Cashier withPriceList: priceList.

	cart := Cart withCatalog: catalog.
	cart add: aBook. 
	cart add: aBook.
	cart add: anotherBook.
	
	self assert: (cashier checkout: cart) = 5. ! !

!TusLibrosTest methodsFor: '2 - Cashier Test' stamp: 'JuanManuelTastzian 6/3/2013 20:17'!
test3BooksNotPresentInPriceListCantBeCheckedOut

	"Me fijo que los libros del carrito se encuentren en la lista de precios antes de hacer checkout"

	| cashier aBook catalog priceList anotherBook cart priceBook priceAnotherBook |

	aBook := 'aBook'.
	anotherBook := 'anotherBook'.

	catalog := Bag new.
	catalog add: aBook.
	catalog add: anotherBook .

	priceBook := Association key: aBook value: 1.
	priceAnotherBook := Association key: anotherBook value: 3.

	priceList := Dictionary new.
	priceList add: priceBook.

	cart := Cart withCatalog: catalog.
	cart add: aBook. 
	cart add: anotherBook.

	cashier := Cashier withPriceList: priceList.

	self should: [cashier checkout:  cart ]
		raise: Error 
		withExceptionDo: [ :anError | 
			self assert: anError description = Cashier bookNotInPriceList ]! !

!TusLibrosTest methodsFor: '1 - Cart Test' stamp: 'JuanManuelTastzian 5/29/2013 20:59'!
test3aBookShouldBeListedAfterBeingAddedToACart
	| cart book catalog |
	
	book := 'aBook'.
	catalog := Bag new.
	catalog add: book.
	cart := Cart withCatalog: catalog.
	
	cart add: book.
	self assert: (cart includes: book)! !

!TusLibrosTest methodsFor: '2 - Cashier Test' stamp: 'JuanManuelTastzian 6/3/2013 21:46'!
test4CashierCantCheckOutExpiredCard

	"Debera andar as como est, pero no se por que no lo hace
	Hay algn problema con la evaluacin del bloque y eso"

	| aBook catalog priceBook priceList cart cashier expiredCreditCard |
	aBook := 'aBook'.

	catalog := Bag new.
	catalog add: aBook.
	
	priceBook := Association key: aBook value: 1.
	
	priceList := Dictionary new.
	priceList add: priceBook.
	
	cart := Cart withCatalog: catalog.
	cart add: aBook. 
	
	cashier := Cashier withPriceList: priceList.
	
	expiredCreditCard := CreditCard newExpiredCreditCard.

	self should: [cashier checkout: cart with: expiredCreditCard ]
		raise: Error 
		withExceptionDo: [ :anError | 
			self assert: anError description = Cashier expiredCreditCard ]! !

!TusLibrosTest methodsFor: '1 - Cart Test' stamp: 'JuanManuelTastzian 5/29/2013 20:59'!
test4aCartHasOnlyOneBookAfterAddingABookToAnEmptyCart
	| cart book catalog |
	
	book := 'aBook'.
	catalog := Bag new.
	catalog add: book.
	cart := Cart withCatalog: catalog.
	
	cart add: book.
	self assert: (cart includes: book).
	self assert: cart size = 1! !

!TusLibrosTest methodsFor: '1 - Cart Test' stamp: 'JuanManuelTastzian 5/29/2013 21:02'!
test5aBookCanBeAddedMoreThanOnce
	| cart book anotherBook catalog |
	
	book := 'aBook'.
	anotherBook := 'anotherBook'.
	catalog := Bag new.
	catalog add: book.
	catalog add: anotherBook.
	
	cart := Cart withCatalog: catalog.
	
	cart add: book.
	cart add: book.
	cart add: anotherBook.
	
	self assert: (cart includes: book).
	self assert: cart size = 3.
	self assert: (cart numberOfUnitsAddedOf: book) = 2.
	self assert: (cart numberOfUnitsAddedOf: anotherBook) = 1.! !

!TusLibrosTest methodsFor: '1 - Cart Test' stamp: 'JuanManuelTastzian 5/29/2013 20:52'!
test6booksThatAreNotOnTheCatalogCantBeAddedToCart
	| cart book |
	
	cart := Cart newEmptyCart.
	book := 'aBook'.
	
	self should: [cart add: book ]
		raise: Error 
		withExceptionDo: [ :anError | 
			self assert: anError description = Cart productNotInCatalog ]! !

!TusLibrosTest methodsFor: '1 - Cart Test' stamp: 'JuanManuelTastzian 5/29/2013 20:39'!
test7onlyBooksFromCatalogCanBeAddedToCart
	| cart book catalog |
	
	book := 'aBook'.
	catalog := Bag new.
	catalog add: book.
	cart := Cart withCatalog: catalog.
	
	cart add: book.
	
	self assert: (cart includes: book).
	self assert: cart size = 1.! !
